"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDslAppGenerator = void 0;
const tslib_1 = require("tslib");
const path_1 = tslib_1.__importDefault(require("path"));
const types_1 = require("../../../types");
const types_2 = require("../../../types");
const utils_1 = require("../../../utils");
const fileSystem_1 = require("../../actions/fileSystem");
const npm_1 = require("../../actions/npm");
const seed_1 = require("../../actions/seed");
const normalizers_1 = require("../../normalizers");
const dsl_1 = tslib_1.__importDefault(require("../../prompts/dsl"));
const filesystem_1 = tslib_1.__importDefault(require("../../prompts/filesystem"));
const tasks_1 = require("../../tasks");
const applyRuntimeArgs = (prevAns = {}) => {
    if (!prevAns.appDirectory) {
        prevAns.appDirectory = utils_1.argv.appDirectory || utils_1.env.GENESISLCAP_APP_DIRECTORY;
    }
    if (!prevAns.appName) {
        prevAns.appName = utils_1.argv.appName || utils_1.env.GENESISLCAP_APP_NAME;
    }
    if (prevAns.forceOverwrite === undefined) {
        prevAns.forceOverwrite = utils_1.argv.forceOverwrite || utils_1.env.GENESISLCAP_FORCE_OVERWRITE;
    }
    return prevAns;
};
exports.createDslAppGenerator = {
    description: null,
    prompts(inquirer, prevAns = {}) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            prevAns = applyRuntimeArgs(prevAns);
            const seedList = (yield (0, tasks_1.downloadSeedList)('apps')).filter((seed) => { var _a; return (_a = seed.tags) === null || _a === void 0 ? void 0 : _a.includes(types_1.DslTag); });
            if (seedList.length === 0) {
                throw new Error(`No compatible ${types_2.dslName.toLowerCase()} seeds found, unable to proceed.`);
            }
            let { appDirectory, appName, appSeed } = yield (0, dsl_1.default)(inquirer, prevAns, seedList);
            appSeed = yield (0, normalizers_1.seedSelectionNormalizer)(inquirer, appSeed);
            const { forceOverwrite } = yield (0, filesystem_1.default)(inquirer, prevAns, appDirectory, appName);
            return {
                appDirectory,
                appName,
                appSeed,
                forceOverwrite,
            };
        });
    },
    actions: (answers) => [
        {
            type: fileSystem_1.FileSystemActionType.createPath,
            get answerKey() {
                return 'targetDirectory';
            },
            get paths() {
                return [answers.appDirectory, answers.appName];
            },
        },
        {
            type: fileSystem_1.FileSystemActionType.createDirectory,
            get directory() {
                return answers.targetDirectory;
            },
        },
        {
            type: seed_1.SeedActionType.create,
            get directory() {
                return answers.targetDirectory;
            },
            get seed() {
                return answers.appSeed;
            },
        },
        `Installing dependencies`,
        {
            type: npm_1.NpmActionType.install,
            get directory() {
                return path_1.default.join(answers.targetDirectory, '.genx');
            },
        },
    ],
};
